/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irckactr.c
 *
 * Purpose: This file contains the functions for checking the
 *          mandatory fields of access control request PDU and access control
 *          response PDU.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *              ir_check_access_control_request()
 *              ir_check_access_control_response()
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpaccctr.c,v 1.2 1994/02/11 13:37:04 irsuprt Exp irsuprt $
 * $Log: tpaccctr.c,v $
 *
 *
 *****************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>

#include "ir.h"
#include "irconst.h"
#include "irerror.h"

extern int errno;

/*****************************************************************************
 *
 * Function: ir_check_access_control_request
 *
 * Purpose: This function checks the mandatory fields of access control request
 *          PDU.
 *
 * Input Arguments:
 *          PDU - the Init Request PDU or the Access Control Response PDU
 *
 * Return values: SUCCESS - if the mandatory fields are valid
 *                FAILED  - if any of the mandatory fields is invalid
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/
ir_check_access_control_request(PDU)
    APDU *PDU;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_check_access_control_request\n");
#endif

    /* Checking the older version of Access Control Request */
    if (PDU->a.accessControlReq->securityChallenge->choiceId == ACCESSCONTROLREQCHOICE_OLDWAY)
    {

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "ir_check_access_control_request: access control OLDWAY\n");
#endif

        /* Using the old data structure. */
        if (PDU->a.accessControlReq->securityChallenge->a.oldway->octetLen <= 0)
        {
            sprintf(ERR_STR, "SECURITY CHALLENGE LENGTH IS <= 0");
            display_error("KERNEL", DISPLAY_USER_ERROR, -1);
            return (FAILED);
        }

        /* Only password, all users uses one password. */
        if (strcmp(PDU->a.accessControlReq->securityChallenge->a.oldway->octs, "") == 0)
        {
            strcpy(ERR_STR, "SECURITY CHALLENGE IS EMPTY");
            display_error("KERNEL", DISPLAY_USER_ERROR, -1);
            return (FAILED);
        }
    }
    else
    {

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "ir_check_access_control_request: access control NEWWAY\n");
#endif

        /* Using the new data structure. */
        if (PDU->a.accessControlReq->securityChallenge->a.newway->encoding->choiceId == EXTERNALCHOICE_SINGLE_ASN1_TYPE)
        {
            if (PDU->a.accessControlReq->securityChallenge->a.newway->encoding->a.single_ASN1_type->octetLen <= 0)
            {
				sprintf(ERR_STR, "SECURITY CHALLENGE LENGTH IS <= 0");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
            }

            if (strcmp(PDU->a.accessControlReq->securityChallenge->a.newway->encoding->a.single_ASN1_type->octs, "") == 0)
			{
				strcpy(ERR_STR, "SECURITY CHALLENGE IS EMPTY");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
			}
        }
        else if (PDU->a.accessControlReq->securityChallenge->a.newway->encoding->choiceId == EXTERNALCHOICE_OCTET_ALIGNED)
        {
            if (PDU->a.accessControlReq->securityChallenge->a.newway->encoding->a.octet_aligned->octetLen <= 0)
            {
				sprintf(ERR_STR, "SECURITY CHALLENGE LENGTH IS <= 0");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
            }

            if (strcmp(PDU->a.accessControlReq->securityChallenge->a.newway->encoding->a.octet_aligned->octs, "") == 0)
            {
				strcpy(ERR_STR, "SECURITY CHALLENGE IS EMPTY");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
            }
        }
	}

	return(SUCCESS);

} /* End of ir_check_access_control_request */

/*****************************************************************************
 *
 * Function: ir_check_access_control_response
 *
 * Purpose: This function checks the mandatory fields of access control response
 *          PDU.
 *
 * Input Arguments:
 *          PDU - the Init Request PDU or the Access Control Response PDU
 *
 * Return values: SUCCESS - if the mandatory fields are valid
 *                FAILED  - if any of the mandatory fields is invalid
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/
ir_check_access_control_response(PDU)
    APDU *PDU;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_check_access_control_response\n");
#endif

    /* Checking the older version of Access Control Request */
	if (PDU->a.accessControlResp->securityChallengeResponse->choiceId == ACCESSCONTROLRESPCHOICE_OLDWAY)
    {
        /* Using the old data structure. */
        if (PDU->a.accessControlResp->securityChallengeResponse->a.oldway->octetLen <= 0)
        {
            sprintf(ERR_STR, "PASSWORD LENGTH IS <= 0");
            display_error("KERNEL", DISPLAY_USER_ERROR, -1);
            return (FAILED);
        }

        /* Only password, all users uses one password. */
        if (strcmp(PDU->a.accessControlResp->securityChallengeResponse->a.oldway->octs, "") == 0)
        {
            strcpy(ERR_STR, "PASSWORD IS EMPTY");
            display_error("KERNEL", DISPLAY_USER_ERROR, -1);
            return (FAILED);
        }
    }
    else
    {
        /* Using the new data structure. */
        if (PDU->a.accessControlResp->securityChallengeResponse->a.newway->encoding->choiceId == EXTERNALCHOICE_SINGLE_ASN1_TYPE)
        {
            if (PDU->a.accessControlResp->securityChallengeResponse->a.newway->encoding->a.single_ASN1_type->octetLen <= 0)
            {
				sprintf(ERR_STR, "PASSWORD LENGTH IS <= 0");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
            }

            if (strcmp(PDU->a.accessControlResp->securityChallengeResponse->a.newway->encoding->a.single_ASN1_type->octs, "") == 0)
			{
				strcpy(ERR_STR, "PASSWORD IS EMPTY");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
			}
        }
        else if (PDU->a.accessControlResp->securityChallengeResponse->a.newway->encoding->choiceId == EXTERNALCHOICE_OCTET_ALIGNED)
        {
            if (PDU->a.accessControlResp->securityChallengeResponse->a.newway->encoding->a.octet_aligned->octetLen <= 0)
            {
				sprintf(ERR_STR, "PASSWORD LENGTH IS <= 0");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
            }

            if (strcmp(PDU->a.accessControlResp->securityChallengeResponse->a.newway->encoding->a.octet_aligned->octs, "") == 0)
            {
				strcpy(ERR_STR, "PASSWORD IS EMPTY");
				display_error("KERNEL", DISPLAY_USER_ERROR, -1);
				return (FAILED);
            }
        }
	}

	return(SUCCESS);

} /* End of ir_check_access_control_response */
