/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irstkser.c
 *
 * Purpose: This file contains the stack functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *                  ir_push_service()
 *                  ir_pop_service()
 *                  ir_clear_service_stack()
 *                  ir_view_service()
 *                  ir_find_stack_service()
 *                  ir_add_time_service()
 *                  ir_top_service()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: irstkser.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: irstkser.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <time.h>
#include "ir.h"
#include "irconst.h"

struct stack_service
{
    long number;
    struct stack_service *link;
};

/*****************************************************************************
 *
 * Function: ir_push_service
 *
 * Purpose: Create a new stack node with the parameter value and push it
 *          into the stack.
 *
 * Input Arguments:
 *                  stack_service_head - the service stack list pointer
 *                  value - the service value to be inserted
 *
 * Return values: SUCCESS - successfully inserting the new node to the list
 *                FAILED  - failed to insert the new node to the list
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_push_service(stack_service_head, value)
    struct stack_service **stack_service_head;
    long value;
{

    struct stack_service *new;

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "ir_push_service: push:%ld\n", value);
#endif

    new = (struct stack_service *) malloc(sizeof(struct stack_service));
    if (new == NULL)
        return (FAILED);
    else
    {
        new->number = value;
        new->link = (*stack_service_head);
        (*stack_service_head) = new;
        return (SUCCESS);
    }
}

/*****************************************************************************
 *
 * Function: ir_pop_service
 *
 * Purpose: Pop a service from the stack and free the node.
 *
 * Input Arguments:
 *                  stack_service_head - the service stack list pointer
 *
 * Return values: service value from the top of the stack.
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_pop_service(stack_service_head)
    struct stack_service **stack_service_head;
{

    struct stack_service *old;
    int value;

    if ((*stack_service_head))
    {
        value = (*stack_service_head)->number;
        old = (*stack_service_head);
        (*stack_service_head) = (*stack_service_head)->link;
        free(old);
    }
    else
        value = 0;

    return (value);
}


/*****************************************************************************
 *
 * Function: ir_clear_service_stack
 *
 * Purpose: Release all the nodes of the service stack.
 *
 * Input Arguments:
 *                  stack_service_head - the service stack list pointer
 *
 * Return values: None
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used: ir_pop_service()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
void
ir_clear_service_stack(stack_service_head)
    struct stack_service **stack_service_head;
{
    while ((*stack_service_head))
        ir_pop_service(stack_service_head);
}

/*****************************************************************************
 *
 * Function: ir_view_service
 *
 * Purpose: View the service value of all the nodes on the service stack.
 *
 * Input Arguments:
 *                  stack_service_head - the service stack list pointer
 *
 * Return values: None
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
void
ir_view_service(stack_service_head)
    struct stack_service **stack_service_head;
{
    struct stack_service *ptr;
    struct tm *today_time;

    ptr = (*stack_service_head);
    printf("service stack = ");
    while (ptr)
    {
        printf("%ld=", ptr->number);
        today_time = localtime(&ptr->number);
        printf("%d:%d:%d, ", today_time->tm_hour, today_time->tm_min,
               today_time->tm_sec);

        ptr = ptr->link;
    }
    printf("\n");
}

/*****************************************************************************
 *
 * Function: ir_find_service_stack
 *
 * Purpose: Verify if the service_value is in the list.
 *
 * Input Arguments:
 *                  stack_service_head - the service stack list pointer
 *                  service_value - the service value to be searched
 *
 * Return values: SUCCESS - found in the list
 *                FAILED  - not found
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_find_stack_service(stack_service_head, service_value)
    struct stack_service **stack_service_head;
    int service_value;
{
    struct stack_service *ptr;

    ptr = (*stack_service_head);
    while (ptr)
    {
        if (ptr->number == service_value)
            return (SUCCESS);
        ptr = ptr->link;
    }
    return (FAILED);
}

/*****************************************************************************
 *
 * Function: ir_add_time_service
 *
 * Purpose: Adding the new timeout to all the old nodes so as to keep current.
 *
 * Input Arguments:
 *                  stack_service_head - the service stack list pointer
 *                  timeout - the timeout value to be added
 *
 * Return values: SUCCESS
 *                FAILED
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_add_time_service(stack_service_head, timeout)
    struct stack_service **stack_service_head;
    long timeout;
{
    struct stack_service *ptr;

    ptr = (*stack_service_head)->link;        /* points to the second one */
    while (ptr)
    {
        ptr->number += timeout;
        ptr = ptr->link;
    }
    return (SUCCESS);
}


/*****************************************************************************
 *
 * Function: ir_top_service
 *
 * Purpose: Return the value of the top node without poping it off the stack.
 *
 * Input Arguments:
 *                  stack_service_head - the service stack list pointer
 *
 * Return values: - the value of the top node of the service stack or
 *                - zero if it's empty stack.
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
long
ir_top_service(stack_service_head)
    struct stack_service **stack_service_head;
{
    if ((*stack_service_head))
    {
        return ((*stack_service_head)->number);
    }
    else
        return (0);
}
